@@ -108,12 +108,13 @@ static const PHYSFS_ArchiveInfo *supported_types[] =
     &__PHYSFS_ArchiveInfo_WAD,
 #endif
 
-
     NULL
 };
 
 static const DirFunctions *dirFunctions[] =
 {
+    &__PHYSFS_DirFunctions_DIR,
+
 #if (defined PHYSFS_SUPPORTS_ZIP)
     &__PHYSFS_DirFunctions_ZIP,
 #endif
@@ -138,7 +139,6 @@ static const DirFunctions *dirFunctions[] =
     &__PHYSFS_DirFunctions_WAD,
 #endif
 
-    &__PHYSFS_DirFunctions_DIR,
     NULL
 };
 
@@ -214,7 +214,8 @@ static void __PHYSFS_quick_sort(void *a, PHYSFS_uint32 lo, PHYSFS_uint32 hi,
                 break;
             swapfn(a, i, j);
         } /* while */
-        swapfn(a, i, hi-1);
+        if (i != (hi-1))
+            swapfn(a, i, hi-1);
         __PHYSFS_quick_sort(a, lo, j, cmpfn, swapfn);
         __PHYSFS_quick_sort(a, i+1, hi, cmpfn, swapfn);
     } /* else */
@@ -1090,7 +1091,7 @@ int PHYSFS_setSaneConfig(const char *organization, const char *appName,
         } /* if */
         else
         {
-                no_write = 1;
+            no_write = 1;
         } /* else */
 
         if (no_write)
@@ -1797,6 +1798,8 @@ PHYSFS_sint64 PHYSFS_read(PHYSFS_file *handle, void *buffer,
     FileHandle *h = (FileHandle *) handle->opaque;
 
     BAIL_IF_MACRO(!h->forReading, ERR_FILE_ALREADY_OPEN_W, -1);
+    BAIL_IF_MACRO(objSize == 0, NULL, 0);
+    BAIL_IF_MACRO(objCount == 0, NULL, 0);
     if (h->buffer != NULL)
         return(doBufferedRead(handle, buffer, objSize, objCount));
 
@@ -1830,6 +1833,8 @@ PHYSFS_sint64 PHYSFS_write(PHYSFS_file *handle, const void *buffer,
     FileHandle *h = (FileHandle *) handle->opaque;
 
     BAIL_IF_MACRO(h->forReading, ERR_FILE_ALREADY_OPEN_R, -1);
+    BAIL_IF_MACRO(objSize == 0, NULL, 0);
+    BAIL_IF_MACRO(objCount == 0, NULL, 0);
     if (h->buffer != NULL)
         return(doBufferedWrite(handle, buffer, objSize, objCount));
 
