@@ -22,13 +22,17 @@
 #define assert(x)
 #endif
 
+#if defined(_MSC_VER) || defined(__MINGW32__)
+#include <malloc.h>
+#endif
+
 #ifdef __cplusplus
 extern "C" {
 #endif
 
 
 /* The LANG section. */
-/*  please send questions/translations to Ryan: icculus@clutteredmind.org. */
+/*  please send questions/translations to Ryan: icculus@icculus.org. */
 
 #if (!defined PHYSFS_LANG)
 #  define PHYSFS_LANG PHYSFS_LANG_ENGLISH
