@@ -25,14 +25,12 @@
 
 #if (defined _MSC_VER)
     #define alloca(x) _alloca(x)
-#elif (defined MINGW)  /* scary...hopefully this is okay. */
+#elif (defined __MINGW32__)  /* scary...hopefully this is okay. */
     #define alloca(x) __builtin_alloca(x) 
 #endif
 
-#define LOWORDER_UINT64(pos) (PHYSFS_uint32) \
-    (pos & 0x00000000FFFFFFFF)
-#define HIGHORDER_UINT64(pos) (PHYSFS_uint32) \
-    (((pos & 0xFFFFFFFF00000000) >> 32) & 0x00000000FFFFFFFF)
+#define LOWORDER_UINT64(pos) ((PHYSFS_uint32) (pos & 0xFFFFFFFF))
+#define HIGHORDER_UINT64(pos) ((PHYSFS_uint32) ((pos >> 32) & 0xFFFFFFFF))
 
 /* GetUserProfileDirectory() is only available on >= NT4 (no 9x/ME systems!) */
 typedef BOOL (STDMETHODCALLTYPE FAR * LPFNGETUSERPROFILEDIR) (
@@ -80,8 +78,7 @@ static const char *win32strerror(void)
 {
     static TCHAR msgbuf[255];
     TCHAR *ptr = msgbuf;
-
-    FormatMessage(
+    DWORD rc = FormatMessage(
         FORMAT_MESSAGE_FROM_SYSTEM |
         FORMAT_MESSAGE_IGNORE_INSERTS,
         NULL,
@@ -92,12 +89,15 @@ static const char *win32strerror(void)
         NULL 
     );
 
-        /* chop off newlines. */
+    if (rc == 0)
+        msgbuf[0] = '\0';  /* oh well. Empty string. */
+
+    /* chop off newlines... */
     for (ptr = msgbuf; *ptr; ptr++)
     {
         if ((*ptr == '\n') || (*ptr == '\r'))
         {
-            *ptr = ' ';
+            *ptr = '\0';
             break;
         } /* if */
     } /* for */
@@ -204,7 +204,7 @@ static int determineUserDir(void)
     lib = LoadLibrary("userenv.dll");
     if (lib)
     {
-        /* !!! FIXME: Handle Unicode? */
+        /* If you want Unicode, upgrade to PhysicsFS 1.1.1 or later! */
         GetUserProfileDirectory = (LPFNGETUSERPROFILEDIR)
                               GetProcAddress(lib, "GetUserProfileDirectoryA");
         if (GetUserProfileDirectory)
@@ -850,8 +850,8 @@ PHYSFS_sint64 __PHYSFS_platformWrite(void *opaque, const void *buffer,
 int __PHYSFS_platformSeek(void *opaque, PHYSFS_uint64 pos)
 {
     HANDLE FileHandle = ((win32file *) opaque)->handle;
-    DWORD HighOrderPos;
-    DWORD *pHighOrderPos;
+    LONG HighOrderPos;
+    PLONG pHighOrderPos;
     DWORD rc;
 
     /* Get the high order 32-bits of the position */
@@ -888,7 +888,7 @@ int __PHYSFS_platformSeek(void *opaque, PHYSFS_uint64 pos)
 PHYSFS_sint64 __PHYSFS_platformTell(void *opaque)
 {
     HANDLE FileHandle = ((win32file *) opaque)->handle;
-    DWORD HighPos = 0;
+    LONG HighPos = 0;
     DWORD LowPos;
     PHYSFS_sint64 retval;
 
@@ -972,7 +972,7 @@ int __PHYSFS_platformClose(void *opaque)
 int __PHYSFS_platformDelete(const char *path)
 {
     /* If filename is a folder */
-    if (GetFileAttributes(path) == FILE_ATTRIBUTE_DIRECTORY)
+    if (GetFileAttributes(path) & FILE_ATTRIBUTE_DIRECTORY)
     {
         BAIL_IF_MACRO(!RemoveDirectory(path), win32strerror(), 0);
     } /* if */
